import * as s3 from "aws-cdk-lib/aws-s3";
import { Construct } from "constructs";
/**
 * Well Architected Bucket alarms
 */
export declare enum WaBucketAlarms {
    /**
     * This enum is used to represent an alarm name for 4XX errors e.g. 404 Not Found
     */
    FOURXX_ERRORS_ALARM = "4xxErrorsAlarm",
    /**
     * This enum is used to represent an alarm name for 5XX errors e.g. 500 Internal Server Error
     */
    FIVEXX_ERRORS_ALARM = "5xxErrorsAlarm"
}
/**
 * Well Architected Bucket Properties
 */
export interface WaBucketProps extends s3.BucketProps {
    /**
     * This flag is used to skip the default alarms.
     */
    readonly waDoNotAddDefaultAlarms?: boolean;
}
/**
 * Well Architected S3 Bucket that uses S3_MANAGED encryption, enforces ssl, denies public access
 * is versioned, and uses CloudFront distribution with bucket as origin.
 *
 * ### Default Alarms
 * By default it is configured with FourxxErrorsAlarm and FivexxErrorsAlarm alarms.
 *
 * Note that in freemium version the default alarm does not have any action. However, in premium version
 * it uses the Well Architected Alarm construct, which sets up an alarm action to notify the SNS
 * Topic *AlarmEventsTopic* by default.
 *
 * ##Example
 * Default Usage
 * ```ts
 * new WaBucket(this, "LogicalId", {});
 * ```
 *
 * ##Example
 * Custom Configuration
 * ```ts
 * new WaBucket(this, "LogicalId", {
 *    enforceSSL: false
 * });
 * ```
 *
 * ### Compliance
 * It addresses the following compliance requirements
 * * Blocks public access
 * > PCI, HIPAA, GDPR, APRA, MAS, NIST4
 * * Bucket versioning enabled
 * > PCI, APRA, MAS, NIST4
 * * Only allow secure transport protocols
 * > PCI, APRA, MAS, NIST4
 * * Server side encryption
 * > PCI, HIPAA, GDPR, APRA, MAS, NIST4
 * */
export declare class WaBucket extends s3.Bucket {
    constructor(scope: Construct, id: string, props: WaBucketProps);
    private add4xxErrorsAlarm;
    private add5xxErrorsAlarm;
}
